import re
import string
import os
import sha

def make_error(element):
	msg = """\
<div class="system-message">
	<strong>Error: the <code>JavaDoc</code> macro returned an error:
	%s
	</strong>
</div>
"""
	return msg % element

def render(hdf, env, sourcefile, mode):
	cfg = env.config
	jeldoclet_path = cfg.get('javadoc', 'jeldoclet_path')
	tmp_path = cfg.get('javadoc', 'tmp_path')

	if not jeldoclet_path:
		return make_error('jeldoclet_path not set in config')
	if not tmp_path:
		tmp_path = "/tmp"

	repos = env.get_repository()
	try:
		src = repos.get_node(sourcefile).get_content().read()
	finally:
		repos.close()
	
	FILE = open("%s/trac_javadoc.java"%tmp_path, "w")
	FILE.write(src)
	FILE.close()

	cmd = "javadoc -docletpath \"%s\" -doclet com.jeldoclet.JELDoclet -%s -d \"%s\" -filename trac_javadoc.xml \"%s/trac_javadoc.java\""%(jeldoclet_path,mode,tmp_path,tmp_path)
	#os.system(cmd)
	
	html = ""
	import xml.dom.minidom
	doc = xml.dom.minidom.parse("%s/trac_javadoc.xml"%tmp_path)
	
	def getText(nodelist):
		rc = ""
		for node in nodelist:
			if node.nodeType == node.TEXT_NODE:
				rc = rc + node.data
		return rc

	def getFirstElementText(element,tagname):
		if element.getElementsByTagName(tagname)[0]:
			return getText(element.getElementsByTagName(tagname)[0].childNodes)
		return ""

	for jelclass in doc.getElementsByTagName("jelclass"):
		html += "<h1>"+jelclass.getAttribute("fulltype")+"</h1>"
		html += "<h2>Methods</h2>"
		for method in jelclass.getElementsByTagName("method"):
			html+="<h3>"+method.getAttribute("name")+"</h3><table><tr><td>Description:</td><td>"
			html+=getFirstElementText(method,"description")
			html+="</td></tr><tr><td>Parameters:</td><td>"
			for param in method.getElementsByTagName("param"):
				html+="<i>"+param.getAttribute("fulltype")+"</i> <b>"+param.getAttribute("name")+"</b>"
				if param.getAttribute("comment"):
					html+=" ("+param.getAttribute("comment")+")"
				html+="<br>"
			html+="</td></tr></table>"
	
	return html

def execute(hdf, text, env):
	cfg = env.config
	mode = 'private'
	text = text.split(",")
	if len(text)>1: 
		mode = text[1]
		if not mode in ('private','public','package','protected'):
			return make_error("Second argument is invalid")
	elif len(text)==0: return make_error("JavaDoc expects at least one argument")
	return render(hdf,env,text[0],mode)

